@@ -1,4 +1,4 @@
-/* Copyright © 2007-2015 Jakub Wilk <jwilk@jwilk.net>
+/* Copyright © 2007-2017 Jakub Wilk <jwilk@jwilk.net>
  * Copyright © 2009 Mateusz Turcza
  *
  * This file is part of pdf2djvu.
@@ -57,7 +57,7 @@ Config::Config()
   this->no_render = false;
   this->monochrome = false;
   this->loss_level = 0;
-  this->bg_slices = NULL;
+  this->bg_slices = nullptr;
   this->page_id_template.reset(default_page_id_template("p"));
   this->page_title_template.reset(new string_format::Template("{label}"));
   this->n_jobs = 1;
@@ -75,26 +75,26 @@ static void parse_hyperlinks_options(std::string s, Config::Hyperlinks &options)
   std::vector<std::string> split;
   std::replace(s.begin(), s.end(), '_', '-');
   string::split(s, ',', split);
-  for (std::vector<std::string>::const_iterator it = split.begin(); it != split.end(); it++)
+  for (const std::string &s : split)
   {
-    if (*it == "border-avis")
+    if (s == "border-avis")
     {
       options.border_always_visible = true;
       continue;
     }
-    else if (*it == "no" || *it == "none")
+    else if (s == "no" || s == "none")
     {
       options.extract = false;
       continue;
     }
     else if
     (
-      it->length() == 7 &&
-      (*it)[0] == '#' &&
-      it->find_first_not_of("0123456789abcdefABCDEF", 1) == std::string::npos
+      s.length() == 7 &&
+      s[0] == '#' &&
+      s.find_first_not_of("0123456789abcdefABCDEF", 1) == std::string::npos
     )
     {
-      options.border_color = *it;
+      options.border_color = s;
       continue;
     }
     throw Config::Error(_("Unable to parse hyperlinks options"));
@@ -110,20 +110,20 @@ static void parse_pages(const std::string &s, std::vector< std::pair<int, int> >
 {
   int state = 0;
   int value[2] = { 0, 0 };
-  for (std::string::const_iterator it = s.begin(); it != s.end(); it++)
+  for (char c : s)
   {
-    if (('0' <= *it) && (*it <= '9'))
+    if ('0' <= c && c <= '9')
     {
-      value[state] = value[state] * 10 + (int)(*it - '0');
+      value[state] = value[state] * 10 + (int)(c - '0');
       if (state == 0)
         value[1] = value[0];
     }
-    else if (state == 0 && *it == '-')
+    else if (state == 0 && c == '-')
     {
       state = 1;
       value[1] = 0;
     }
-    else if (*it == ',')
+    else if (c == ',')
     {
       if (value[0] < 1 || value[1] < 1 || value[0] > value[1])
         bad_pages();
@@ -147,7 +147,7 @@ static std::pair<int, int>parse_page_size(const std::string &s)
   int x, y;
   char c;
   stream >> x >> c >> y;
-  if (x > 0 &&  y > 0 && c == 'x' && stream.eof() && !stream.fail())
+  if (x > 0 && y > 0 && c == 'x' && stream.eof() && !stream.fail())
     return std::make_pair(x, y);
   else
     throw Config::Error(_("Unable to parse page size"));
@@ -208,19 +208,19 @@ static void validate_page_id_template(string_format::Template &page_id_template)
   size_t length = page_id.length();
   bool dot_allowed = false;
   bool pm_allowed = false;
-  for (std::string::const_iterator it = page_id.begin(); it != page_id.end(); it++)
+  for (char c : page_id)
   {
-    if (!pm_allowed && (*it == '+' || *it == '-'))
+    if (!pm_allowed && (c == '+' || c == '-'))
     {
       throw Config::Error(_("Page identifier cannot start with a '+' or a '-' character"));
     }
-    if (*it == '.')
+    if (c == '.')
     {
       if (!dot_allowed)
         throw Config::Error(_("Page identifier cannot start with a '.' character or contain two consecutive '.' characters"));
       dot_allowed = false;
     }
-    else if ((*it >= 'a' && *it <= 'z') || (*it >= '0' && *it <= '9') || *it == '_' || *it == '-' || *it == '+')
+    else if (('a' <= c && c <= 'z') || ('0' <= c && c <= '9') || c == '_' || c == '-' || c == '+')
       dot_allowed = true;
     else
       throw Config::Error(_("Page identifier must consist only of letters, digits, '_', '+', '-' and '.' characters"));
@@ -277,46 +277,46 @@ void Config::read_config(int argc, char * const argv[])
   };
   static struct option options [] =
   {
-    { "anti-alias", 0, 0, OPT_ANTIALIAS },
-    { "antialias", 0, 0, OPT_ANTIALIAS }, /* deprecated alias */
-    { "bg-slices", 1, 0, OPT_BG_SLICES },
-    { "bg-subsample", 1, 0, OPT_BG_SUBSAMPLE },
-    { "crop-text", 0, 0, OPT_TEXT_CROP },
-    { "dpi", 1, 0, OPT_DPI },
-    { "fg-colors", 1, 0, OPT_FG_COLORS },
-    { "filter-text", 1, 0, OPT_TEXT_FILTER },
-    { "guess-dpi", 0, 0, OPT_GUESS_DPI },
-    { "help", 0, 0, OPT_HELP },
-    { "hyperlinks", 1, 0, OPT_HYPERLINKS },
-    { "indirect", 1, 0, OPT_INDIRECT },
-    { "jobs", 1, 0, OPT_JOBS },
-    { "lines", 0, 0, OPT_TEXT_LINES },
-    { "loss-level", 1, 0, OPT_LOSS_ANY },
-    { "losslevel", 1, 0, OPT_LOSS_ANY }, /* deprecated alias */
-    { "lossy", 0, 0, OPT_LOSS_100 },
-    { "media-box", 0, 0, OPT_MEDIA_BOX },
-    { "monochrome", 0, 0, OPT_MONOCHROME },
-    { "no-hyperlinks", 0, 0, OPT_NO_HLINKS },
-    { "no-metadata", 0, 0, OPT_NO_METADATA },
-    { "no-nfkc", 0, 0, OPT_TEXT_NO_NFKC },
-    { "no-outline", 0, 0, OPT_NO_OUTLINE },
-    { "no-page-titles", 0, 0, OPT_NO_PAGE_TITLES },
-    { "no-render", 0, 0, OPT_NO_RENDER },
-    { "no-text", 0, 0, OPT_TEXT_NONE },
-    { "output", 1, 0, OPT_OUTPUT },
-    { "page-id-prefix", 1, 0, OPT_PAGE_ID_PREFIX },
-    { "page-id-template", 1, 0, OPT_PAGE_ID_TEMPLATE },
-    { "page-size", 1, 0, OPT_PAGE_SIZE },
-    { "page-title-template", 1, 0, OPT_PAGE_TITLE_TEMPLATE },
-    { "pageid-prefix", 1, 0, OPT_PAGE_ID_PREFIX }, /* deprecated alias */
-    { "pageid-template", 1, 0, OPT_PAGE_ID_TEMPLATE }, /* deprecated alias */
-    { "pages", 1, 0, OPT_PAGES },
-    { "quiet", 0, 0, OPT_QUIET },
-    { "verbatim-metadata", 0, 0, OPT_VERBATIM_METADATA },
-    { "verbose", 0, 0, OPT_VERBOSE },
-    { "version", 0, 0, OPT_VERSION },
-    { "words", 0, 0, OPT_TEXT_WORDS },
-    { NULL, 0, 0, '\0' }
+    { "anti-alias", 0, nullptr, OPT_ANTIALIAS },
+    { "antialias", 0, nullptr, OPT_ANTIALIAS }, /* deprecated alias */
+    { "bg-slices", 1, nullptr, OPT_BG_SLICES },
+    { "bg-subsample", 1, nullptr, OPT_BG_SUBSAMPLE },
+    { "crop-text", 0, nullptr, OPT_TEXT_CROP },
+    { "dpi", 1, nullptr, OPT_DPI },
+    { "fg-colors", 1, nullptr, OPT_FG_COLORS },
+    { "filter-text", 1, nullptr, OPT_TEXT_FILTER },
+    { "guess-dpi", 0, nullptr, OPT_GUESS_DPI },
+    { "help", 0, nullptr, OPT_HELP },
+    { "hyperlinks", 1, nullptr, OPT_HYPERLINKS },
+    { "indirect", 1, nullptr, OPT_INDIRECT },
+    { "jobs", 1, nullptr, OPT_JOBS },
+    { "lines", 0, nullptr, OPT_TEXT_LINES },
+    { "loss-level", 1, nullptr, OPT_LOSS_ANY },
+    { "losslevel", 1, nullptr, OPT_LOSS_ANY }, /* deprecated alias */
+    { "lossy", 0, nullptr, OPT_LOSS_100 },
+    { "media-box", 0, nullptr, OPT_MEDIA_BOX },
+    { "monochrome", 0, nullptr, OPT_MONOCHROME },
+    { "no-hyperlinks", 0, nullptr, OPT_NO_HLINKS },
+    { "no-metadata", 0, nullptr, OPT_NO_METADATA },
+    { "no-nfkc", 0, nullptr, OPT_TEXT_NO_NFKC },
+    { "no-outline", 0, nullptr, OPT_NO_OUTLINE },
+    { "no-page-titles", 0, nullptr, OPT_NO_PAGE_TITLES },
+    { "no-render", 0, nullptr, OPT_NO_RENDER },
+    { "no-text", 0, nullptr, OPT_TEXT_NONE },
+    { "output", 1, nullptr, OPT_OUTPUT },
+    { "page-id-prefix", 1, nullptr, OPT_PAGE_ID_PREFIX },
+    { "page-id-template", 1, nullptr, OPT_PAGE_ID_TEMPLATE },
+    { "page-size", 1, nullptr, OPT_PAGE_SIZE },
+    { "page-title-template", 1, nullptr, OPT_PAGE_TITLE_TEMPLATE },
+    { "pageid-prefix", 1, nullptr, OPT_PAGE_ID_PREFIX }, /* deprecated alias */
+    { "pageid-template", 1, nullptr, OPT_PAGE_ID_TEMPLATE }, /* deprecated alias */
+    { "pages", 1, nullptr, OPT_PAGES },
+    { "quiet", 0, nullptr, OPT_QUIET },
+    { "verbatim-metadata", 0, nullptr, OPT_VERBATIM_METADATA },
+    { "verbose", 0, nullptr, OPT_VERBOSE },
+    { "version", 0, nullptr, OPT_VERSION },
+    { "words", 0, nullptr, OPT_TEXT_WORDS },
+    { nullptr, 0, nullptr, '\0' }
   };
   int optindex, c;
   while (true)
@@ -454,7 +454,7 @@ void Config::read_config(int argc, char * const argv[])
       {
         this->page_id_template.reset(new string_format::Template(optarg));
       }
-      catch (string_format::ParseError)
+      catch (const string_format::ParseError &)
       {
         throw Config::Error(_("Unable to parse page identifier template specification"));
       }
@@ -469,14 +469,14 @@ void Config::read_config(int argc, char * const argv[])
           new string_format::Template(sstream.str())
         );
       }
-      catch (encoding::Error &exc)
+      catch (const encoding::Error &exc)
       {
         throw Config::Error(string_printf(
           _("Unable to convert page title to UTF-8: %s"),
           exc.what())
         );
       }
-      catch (string_format::ParseError)
+      catch (const string_format::ParseError &)
       {
         throw Config::Error(
           _("Unable to parse page title template specification")
