@@ -1,4 +1,4 @@
-/* Copyright © 2007-2016 Jakub Wilk <jwilk@jwilk.net>
+/* Copyright © 2007-2017 Jakub Wilk <jwilk@jwilk.net>
  * Copyright © 2009 Mateusz Turcza
  *
  * This file is part of pdf2djvu.
@@ -72,13 +72,13 @@ public:
 
 static int get_page_for_goto_link(pdf::link::GoTo *goto_link, pdf::Catalog *catalog)
 {
-  std::auto_ptr<pdf::link::Destination> dest;
+  std::unique_ptr<pdf::link::Destination> dest;
   dest.reset(goto_link->getDest());
-  if (dest.get() == NULL)
+  if (dest.get() == nullptr)
     dest.reset(catalog->findDest(goto_link->getNamedDest()));
   else
     dest.reset(dest.release()->copy());
-  if (dest.get() != NULL)
+  if (dest.get() != nullptr)
   {
     int page;
     if (dest->isPageRef())
@@ -165,7 +165,7 @@ public:
     return this->title;
   }
 
-  void set_title(const std::string &title)
+  const std::string & set_title(const std::string &title)
   {
     this->title = title;
     // TODO: issue a warning if the title contains null bytes
@@ -174,6 +174,7 @@ public:
       "\xEF\xBF\xBD"  // U+FFFD REPLACEMENT CHARACTER
     );
     this->title_set = true;
+    return this->title;
   }
 
   const std::string & get_basename() const
@@ -219,15 +220,15 @@ protected:
 
   void clean_files()
   {
-    for (std::vector<Component*>::iterator it = this->components.begin(); it != this->components.end(); it++)
+    for (auto &component : this->components)
     {
-      delete *it;
-      *it = NULL;
+      delete component;
+      component = nullptr;
     }
-    for (std::vector<File*>::iterator it = this->files.begin(); it != this->files.end(); it++)
+    for (auto &file : this->files)
     {
-      delete *it;
-      *it = NULL;
+      delete file;
+      file = nullptr;
     }
   }
 
@@ -258,7 +259,7 @@ public:
   virtual Component &operator[](int n)
   {
     std::vector<Component*>::reference tmpfile_ptr = this->components.at(n - 1);
-    if (tmpfile_ptr == NULL)
+    if (tmpfile_ptr == nullptr)
     {
       this->files[n - 1] = this->create_file(this->get_file_name(n));
       tmpfile_ptr = new Component(*this->files[n - 1]);
@@ -283,7 +284,7 @@ typedef pdf::Renderer MainRenderer;
 class MutedRenderer: public pdf::Renderer
 {
 protected:
-  std::auto_ptr<std::ostringstream> text_comments;
+  std::unique_ptr<std::ostringstream> text_comments;
   std::vector<sexpr::Ref> annotations;
   const ComponentList &page_files;
   bool skipped_elements;
@@ -414,7 +415,7 @@ public:
       if (px + pw < 0 || py + ph < 0 || px >= bitmap_width || py >= bitmap_height)
         return;
     }
-    std::auto_ptr<pdf::NFKC> nfkc;
+    std::unique_ptr<pdf::NFKC> nfkc;
     if (config.text_nfkc)
       nfkc.reset(new pdf::FullNFKC(unistr, length));
     else
@@ -439,7 +440,7 @@ public:
       return;
     double x1, y1, x2, y2;
     pdf::link::Action *link_action = link->getAction();
-    if (link_action == NULL)
+    if (link_action == nullptr)
     {
       debug(1) << _("Warning: Unable to convert link without an action") << std::endl;
       return;
@@ -458,7 +459,7 @@ public:
       {
         page = get_page_for_goto_link(dynamic_cast<pdf::link::GoTo*>(link_action), this->catalog);
       }
-      catch (NoLinkDestination &ex)
+      catch (const NoLinkDestination &ex)
       {
         debug(1) << string_printf(_("Warning: %s"), ex.what()) << std::endl;
         return;
@@ -485,12 +486,10 @@ public:
     case actionJavaScript:
       debug(1) << _("Warning: Unable to convert link with a JavaScript action") << std::endl;
       return;
-#if POPPLER_VERSION >= 1302
     case actionOCGState:
       // L10N: OCG stands for “Optional Content Group” (see PDF Reference v1.7, §4.10.1)
       debug(1) << _("Warning: Unable to convert link with a set-OCG-state action") << std::endl;
       return;
-#endif
     default:
       debug(1) << _("Warning: Unknown link action") << std::endl;
       return;
@@ -598,15 +597,15 @@ public:
     std::string texts = this->text_comments->str();
     if (config.text_filter_command_line.length() > 0)
       texts = Command::filter(config.text_filter_command_line, texts);
-    for (std::string::iterator it = texts.begin(); it != texts.end(); it++)
-      switch (*it)
+    for (char &c : texts)
+      switch (c)
       {
       case '\x01':
-        *it = '#'; break;
+        c = '#'; break;
       case '\x02':
-        *it = 'T'; break;
+        c = 'T'; break;
       case '\x03':
-        *it = 'x'; break;
+        c = 'x'; break;
       }
     return texts;
   }
@@ -682,14 +681,14 @@ void pdf_outline_to_djvu_outline(pdf::Object *node, pdf::Catalog *catalog,
       int page;
       {
         pdf::OwnedObject destination;
-        std::auto_ptr<pdf::link::Action> link_action;
+        std::unique_ptr<pdf::link::Action> link_action;
         if (!pdf::dict_lookup(current, "Dest", &destination)->isNull())
           link_action.reset(pdf::link::Action::parseDest(&destination));
         else if (!pdf::dict_lookup(current, "A", &destination)->isNull())
           link_action.reset(pdf::link::Action::parseAction(&destination));
         else
           throw NoPageForBookmark();
-        if (link_action.get() == NULL || link_action->getKind() != actionGoTo)
+        if (link_action.get() == nullptr || link_action->getKind() != actionGoTo)
           throw NoPageForBookmark();
         try
         {
@@ -698,7 +697,7 @@ void pdf_outline_to_djvu_outline(pdf::Object *node, pdf::Catalog *catalog,
             catalog
           );
         }
-        catch (NoLinkDestination &ex)
+        catch (const NoLinkDestination &ex)
         {
           throw NoPageForBookmark();
         }
@@ -711,7 +710,7 @@ void pdf_outline_to_djvu_outline(pdf::Object *node, pdf::Catalog *catalog,
         pdf_outline_to_djvu_outline(&current, catalog, djvu_outline_item, page_files, depth + 1);
       }
     }
-    catch (BookmarkError &ex)
+    catch (const BookmarkError &ex)
     {
       debug(1) << string_printf(_("Warning: %s"), ex.what()) << std::endl;
     }
@@ -751,7 +750,7 @@ static void add_meta_date(const char *key, const pdf::Timestamp &value, std::ost
     sexpr::Ref sexpr = sexpr::string(value.format(' '));
     stream << key << "\t" << sexpr << std::endl;
   }
-  catch (pdf::Timestamp::Invalid)
+  catch (const pdf::Timestamp::Invalid &)
   {
     debug(1) << string_printf(_("Warning: metadata[%s] is not a valid date"), key) << std::endl;
   }
@@ -769,8 +768,8 @@ private:
   TemporaryComponentList(const TemporaryComponentList&); // not defined
   TemporaryComponentList& operator=(const TemporaryComponentList&); // not defined
 protected:
-  std::auto_ptr<const TemporaryDirectory> directory;
-  std::auto_ptr<TemporaryFile> shared_ant_file;
+  std::unique_ptr<const TemporaryDirectory> directory;
+  std::unique_ptr<TemporaryFile> shared_ant_file;
 
   virtual File *create_file(const std::string &page_id)
   {
@@ -841,7 +840,6 @@ class DjVm
 {
 protected:
   std::set<std::string> known_ids;
-  std::set<std::string> known_titles;
   class DuplicateId : public std::runtime_error
   {
   public:
@@ -849,13 +847,6 @@ protected:
     : std::runtime_error(string_printf(_("Duplicate page identifier: %s"), id.c_str()))
     { }
   };
-  class DuplicateTitle : public std::runtime_error
-  {
-  public:
-    DuplicateTitle(const std::string &id)
-    : std::runtime_error(string_printf(_("Duplicate page title: %s"), id.c_str()))
-    { }
-  };
   void remember(const Component &component);
 public:
   virtual void add(const Component &component) = 0;
@@ -877,13 +868,6 @@ void DjVm::remember(const Component &component)
   if (this->known_ids.count(id) > 0)
     throw DuplicateId(id);
   this->known_ids.insert(id);
-  const std::string &title = component.get_title();
-  if (title.length() > 0)
-  {
-    if (this->known_titles.count(title) > 0)
-      throw DuplicateTitle(title);
-    this->known_titles.insert(title);
-  }
 }
 
 class IndirectDjVm;
@@ -894,8 +878,8 @@ protected:
   size_t size;
   File &output_file;
   DjVuCommand converter;
-  std::auto_ptr<IndirectDjVm> indirect_djvm;
-  std::auto_ptr<TemporaryFile> index_file;
+  std::unique_ptr<IndirectDjVm> indirect_djvm;
+  std::unique_ptr<TemporaryFile> index_file;
 public:
   explicit BundledDjVm(File &output_file)
   : size(0),
@@ -916,7 +900,7 @@ protected:
   File &index_file;
   std::vector<Component> components;
   bool needs_shared_ant;
-  std::auto_ptr<std::ostringstream> outline_stream;
+  std::unique_ptr<std::ostringstream> outline_stream;
   class UnexpectedDjvuSedOutput : public std::runtime_error
   {
   public:
@@ -945,7 +929,7 @@ public:
   {
     if (!outline)
     {
-      this->outline_stream.reset(NULL);
+      this->outline_stream.reset(nullptr);
       return;
     }
     this->outline_stream.reset(new std::ostringstream);
@@ -956,7 +940,7 @@ public:
   {
     size_t size = this->components.size();
     {
-      /* Using ``djvused`` to add shared annotations to a indirect multi-page
+      /* Using ``djvused`` to add shared annotations to an indirect multi-page
        * document could be unacceptably slow:
        * https://sourceforge.net/p/djvu/bugs/100/
        *
@@ -1021,7 +1005,7 @@ public:
 
 void BundledDjVm::add(const Component &component)
 {
-  if (this->index_file.get() == NULL)
+  if (this->index_file.get() == nullptr)
   {
     std::ostringstream stream;
     stream << component << ".djvu-index";
@@ -1056,7 +1040,7 @@ void BundledDjVm::commit()
     << *this->index_file
     << this->output_file;
   this->converter(); // djvmcvt -b <output-djvu-file> <index-djvu-file>
-  this->index_file.reset(NULL);
+  this->index_file.reset(nullptr);
 }
 
 void IndirectDjVm::create_bare(const std::vector<Component> &components)
@@ -1067,7 +1051,7 @@ void IndirectDjVm::create_bare(const std::vector<Component> &components)
 void IndirectDjVm::create(const std::vector<Component> &components, bool bare)
 {
   size_t size = components.size();
-  this->index_file.reopen(true); // (re)open and truncate
+  this->index_file.reopen(File::trunc); // (re)open and truncate
   this->index_file.write("AT&TFORM\0\0\0\0DJVMDIRM\0\0\0\0\1", 25);
   bool shared_ant = !bare && this->needs_shared_ant;
   for (int i = 1; i >= 0; i--)
@@ -1078,14 +1062,14 @@ void IndirectDjVm::create(const std::vector<Component> &components, bool bare)
       bzz_file.write("\0\0", 3);
     if (shared_ant)
       bzz_file << '\3';
-    for (std::vector<Component>::const_iterator it = components.begin(); it != components.end(); it++)
-      bzz_file << (it->get_title().length() == 0 ? '\001' : '\101');
+    for (const Component &component : components)
+      bzz_file << (component.get_title().length() == 0 ? '\001' : '\101');
     if (shared_ant)
       bzz_file << djvu::shared_ant_file_name << '\0';
-    for (std::vector<Component>::const_iterator it = components.begin(); it != components.end(); it++)
+    for (const Component &component : components)
     {
-      bzz_file << it->get_basename() << '\0';
-      const std::string &title = it->get_title();
+      bzz_file << component.get_basename() << '\0';
+      const std::string &title = component.get_title();
       if (title.length() == 0)
         continue;
       bzz_file << title << '\0';
@@ -1150,7 +1134,7 @@ static int calculate_dpi(pdf::Document &doc, int n, bool crop)
         << std::endl;
       return calculate_dpi(guess);
     }
-    catch (pdf::dpi::NoGuess)
+    catch (const pdf::dpi::NoGuess &)
     {
       debug(2) << _("unable to guess resolution") << std::endl;
     }
@@ -1167,9 +1151,9 @@ static int calculate_dpi(pdf::Document &doc, int n, bool crop)
       int_dpi = djvu::max_dpi;
     else
       int_dpi = static_cast<int>(dpi);
-      debug(2)
-        << string_printf(_("estimated resolution: %d dpi"), int_dpi)
-        << std::endl;
+    debug(2)
+      << string_printf(_("estimated resolution: %d dpi"), int_dpi)
+      << std::endl;
     return int_dpi;
   }
   else
@@ -1219,7 +1203,7 @@ static int xmain(int argc, char * const argv[])
   {
     config.read_config(argc, argv);
   }
-  catch (const Config::NeedVersion)
+  catch (const Config::NeedVersion &)
   {
     error_log << get_multiline_version();
     exit(0);
@@ -1258,11 +1242,15 @@ static int xmain(int argc, char * const argv[])
   int n_pages = document_map.get_n_pages();
   PageMap page_map;
   std::vector<int> page_numbers;
-  std::auto_ptr<const Directory> output_dir;
-  std::auto_ptr<File> output_file;
-  std::auto_ptr<DjVm> djvm;
-  std::auto_ptr<ComponentList> page_files;
-  std::auto_ptr<Quantizer> quantizer;
+  std::unique_ptr<const Directory> output_dir;
+  std::unique_ptr<File> output_file;
+  /* `page_files` has to be declared before `djvm`;
+   * otherwise temporary files could be removed in the wrong order:
+   * https://github.com/jwilk/pdf2djvu/issues/114
+   */
+  std::unique_ptr<ComponentList> page_files;
+  std::unique_ptr<DjVm> djvm;
+  std::unique_ptr<Quantizer> quantizer;
   djvu::Outline djvu_outline;
   if (config.monochrome)
     quantizer.reset(new DummyQuantizer(config));
@@ -1310,18 +1298,18 @@ static int xmain(int argc, char * const argv[])
       /* For compatibility reasons, check if it's a directory: */
       output_dir.reset(new Directory(config.output));
     }
-    catch (OSError &no_such_directory_exception)
+    catch (const OSError &no_such_directory_exception)
     {
       bool config_output_not_a_dir = false;
       try
       {
         throw;
       }
-      catch (NoSuchFileOrDirectory &)
+      catch (const NoSuchFileOrDirectory &)
       {
         config_output_not_a_dir = true;
       }
-      catch (NotADirectory &)
+      catch (const NotADirectory &)
       {
         config_output_not_a_dir = true;
       }
@@ -1343,10 +1331,8 @@ static int xmain(int argc, char * const argv[])
   }
   if (config.pages.size() == 0)
     config.pages.push_back(std::make_pair(1, n_pages));
-  for (
-    std::vector< std::pair<int, int> >::iterator page_range = config.pages.begin();
-    page_range != config.pages.end(); page_range++)
-  for (int n = page_range->first; n <= n_pages && n <= page_range->second; n++)
+  for (const std::pair<int, int> &page : config.pages)
+  for (int n = page.first; n <= n_pages && n <= page.second; n++)
   {
     static int i = 1;
     if (page_map.get(n, 0))
@@ -1355,22 +1341,35 @@ static int xmain(int argc, char * const argv[])
     page_numbers.push_back(n);
     i++;
   }
-  for (std::vector<int>::const_iterator np = page_numbers.begin(); np != page_numbers.end(); np++)
   {
-    Component &component = (*page_files)[*np];
-    component.set_title(
-       page_files->get_title(*np, document_map.get(*np).label)
-    );
-    *djvm << component;
+    std::map<std::string, size_t> known_titles;
+    for (int np : page_numbers)
+    {
+      Component &component = (*page_files)[np];
+      const std::string &title = component.set_title(
+         page_files->get_title(np, document_map.get(np).label)
+      );
+      if (title.length() > 0)
+      {
+        size_t n = ++known_titles[title];
+        if (n >= 2)
+        {
+          if (n == 2)
+            debug(1) << string_printf(_("Warning: Ignoring duplicate page title: %s"), title.c_str()) << std::endl;
+          component.set_title("");
+        }
+      }
+      *djvm << component;
+    }
   }
 
   if (page_numbers.size() == 0)
     throw Config::NoPagesSelected();
 
-  std::auto_ptr<MainRenderer> out1;
-  std::auto_ptr<MutedRenderer> outm, outs;
-  std::auto_ptr<pdf::Document> doc;
-  const char *doc_filename = NULL;
+  std::unique_ptr<MainRenderer> out1;
+  std::unique_ptr<MutedRenderer> outm, outs;
+  std::unique_ptr<pdf::Document> doc;
+  const char *doc_filename = nullptr;
 
   bool crop = !config.use_media_box;
   debug(0)++;
@@ -1402,17 +1401,15 @@ static int xmain(int argc, char * const argv[])
         outs->start_doc(doc.get());
       }
     }
-    assert(doc.get() != NULL);
-    assert(out1.get() != NULL);
-    assert(outm.get() != NULL);
+    assert(doc.get() != nullptr);
+    assert(out1.get() != nullptr);
+    assert(outm.get() != nullptr);
     if (!config.monochrome)
-      assert(outs.get() != NULL);
+      assert(outs.get() != nullptr);
     Component &component = (*page_files)[n];
     #pragma omp critical
     {
       debug(1) << string_printf(_("page #%d -> #%d"), n, page_map.get(n));
-      if (config.n_jobs != 1)
-        debug(2) << ":";
       debug(1) << std::endl;
     }
 #if _OPENMP
@@ -1432,7 +1429,7 @@ static int xmain(int argc, char * const argv[])
       /* When the Splash backend runs out of memory,
        * it produces a 1x1 bitmap without signalling an error in any way
        * (other than printing “Out of memory” on stderr).
-       * https://bitbucket.org/jwilk/pdf2djvu/issue/107
+       * https://github.com/jwilk/pdf2djvu/issues/107
        */
       errno = ENOMEM;
       throw_posix_error("");
@@ -1572,9 +1569,8 @@ static int xmain(int argc, char * const argv[])
           ppm_file << "P6 " << bg_width << " " << bg_height << " 255" << std::endl;
           for (int y = 0; y < bg_height; y++)
           for (int x = 0; x < bg_width; x++)
-          for (int i = 0; i < 3; i++)
+          for (char c : background_color)
           {
-            char c = background_color[i];
             ppm_file.write(&c, 1);
           }
           ppm_file.close();
@@ -1614,8 +1610,8 @@ static int xmain(int argc, char * const argv[])
       debug(3) << _("extracting annotations") << std::endl;
       const std::vector<sexpr::Ref> &annotations = outm->get_annotations();
       sed_file << "select 1" << std::endl << "set-ant" << std::endl;
-      for (std::vector<sexpr::Ref>::const_iterator it = annotations.begin(); it != annotations.end(); it++)
-        sed_file << *it << std::endl;
+      for (const sexpr::Ref &annotation : annotations)
+        sed_file << annotation << std::endl;
       sed_file << "." << std::endl;
       outm->clear_annotations();
     }
@@ -1642,12 +1638,12 @@ static int xmain(int argc, char * const argv[])
   /* These exception handlers duplicate the ones in main(), for the sake of OMP.
    * They should be kept in sync.
    */
-  catch (std::ios_base::failure &ex)
+  catch (const std::ios_base::failure &ex)
   {
     error_log << string_printf(_("Input/output error (%s)"), ex.what()) << std::endl;
     exit(2);
   }
-  catch (std::runtime_error &ex)
+  catch (const std::runtime_error &ex)
   {
     error_log << ex << std::endl;
     exit(1);
@@ -1667,7 +1663,7 @@ static int xmain(int argc, char * const argv[])
         {
           xmp_bytes = xmp::transform(xmp_bytes, metadata);
         }
-        catch (xmp::Error &ex)
+        catch (const xmp::Error &ex)
         {
           debug(1) << string_printf(_("Warning: %s"), ex.what()) << std::endl;
         }
@@ -1710,7 +1706,7 @@ static int xmain(int argc, char * const argv[])
         ExistingFile shared_ant(*output_dir, djvu::shared_ant_file_name);
         djvu_size += shared_ant.size();
       }
-      catch (std::ios_base::failure &ex)
+      catch (const std::ios_base::failure &ex)
       {
         /* Let's assume that there are no shared annotations. */
       }
@@ -1744,12 +1740,12 @@ try
   xmain(argc, argv);
 }
 /* Please keep the exception handlers in sync with the ones in xmain(). */
-catch (std::ios_base::failure &ex)
+catch (const std::ios_base::failure &ex)
 {
   error_log << string_printf(_("Input/output error (%s)"), ex.what()) << std::endl;
   exit(2);
 }
-catch (std::runtime_error &ex)
+catch (const std::runtime_error &ex)
 {
   error_log << ex << std::endl;
   exit(1);
