@@ -1,6 +1,6 @@
 # encoding=UTF-8
 
-# Copyright © 2015 Jakub Wilk <jwilk@jwilk.net>
+# Copyright © 2015-2017 Jakub Wilk <jwilk@jwilk.net>
 #
 # This file is part of pdf2djvu.
 #
@@ -13,21 +13,22 @@
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # General Public License for more details.
 
+import re
+
 from tools import (
     case,
-    re,
 )
 
 class test(case):
 
-    # Bug: https://bitbucket.org/jwilk/pdf2djvu/issues/112
-    # + fixed in 0.9.2 [a64d38473b5e]
+    # Bug: https://github.com/jwilk/pdf2djvu/issues/112
+    # + fixed in 0.9.2 [74a8915ca7602adfd0e71c7d329de2265eec4532]
 
     def test(self):
         r = self.pdf2djvu()
         r.assert_(
-            stderr=re(''),
-            stdout=re(''),  # https://bugs.freedesktop.org/show_bug.cgi?id=81513
+            stderr=re.compile(''),
+            stdout=re.compile(''),  # https://bugs.freedesktop.org/show_bug.cgi?id=81513
         )
 
 # vim:ts=4 sts=4 sw=4 et
